   # This script uses BRugs to run the openBUGS model for estimating migration rates from geneotype mixtures
   # The method is published in Broquet, Yearsley et al (2009) Molecular Ecology
   
   #rm(list=ls()) # Remove variables from workspace
   
   source('writeImigData.r')  
  # Read in the Fstat data and format it for openBUGS
  imig = writeImigData()
  numPops = length(imig$demeNames)
  
  # The datafiles generates can be used directly with openBUGS
  # The code below is to use the model with the BRugs interface
  # Using openBUGS directly is usually more robust (and faster)  
  ###############################################################
  
  source('imig_params.r')
  if (runBRugs) {
    # Check to see if required parameters have been defined
    if (length(ls(pattern='paramSave'))==0) {
      warning('paramSave not defined in the file imig_params.r')
    }
    if (length(ls(pattern='thinParam'))==0) {
      warning('thinParam not defined in the file imig_params.r')
    }
    if (length(ls(pattern='burninIters'))==0) {
      warning('burninIters not defined in the file imig_params.r')
    }
    if (length(ls(pattern='datarecordIters'))==0) {
      warning('datarecordIters not defined in the file imig_params.r')
    }
    if (length(ls(pattern='overdisperseInits'))==0) {
      warning('overdisperseInits not defined in the file imig_params.r')
    }
  
    
    require(BRugs,quiet=TRUE)
    modelFile = 'imig_model.bug'
    initFile = vector('list',length=numPops)
    # Loop through each population, read data and run model
    for ( focalPop in 1:numPops ) {
      # Now run Bayesian model
      modelCheck(modelFile)      # Check model syntax
      dataFile <- paste("imigModel_FocalPop",imig$demeNames[focalPop],"_Data.txt",sep="")
      modelData(dataFile)        # Load in model data
      modelCompile(numChains=nChain) # Compile model
      for (chain in 1:nChain) {
        initFile[[chain]] <- paste("imigModel_FocalPop",imig$demeNames[focalPop],"_Inits",chain,".txt",sep="")
        modelInits(initFile[[chain]], chainNum=chain)
      }
      
      ########################################
      if (overdisperseInits) {
        # Run the chain for a bit and then sample the distribution
        # to getting an overrelaxed starting distribution
        modelUpdate(numUpdates=burninIters, thin=1, overRelax = FALSE)
        # Set variable to record output from MCMC
        samplesSet(c("phi", "mu.phi"))
        modelUpdate(numUpdates=datarecordIters, thin=10, overRelax = FALSE)
  
        # Sample the new over dispersed initial values
        stats <- samplesStats("phi")
        mean.phi <- stats$mean
        phiupperCI <- stats$val97.5pc
        philowerCI <- stats$val2.5pc
        SD.phi <- (phiupperCI-philowerCI)/2/1.96
  
        stats <- samplesStats("mu.phi")
        mean.mu <- stats$mean
        mu.phiupperCI <- stats$val97.5pc
        mu.philowerCI <- stats$val2.5pc
        SD.mu <- (mu.phiupperCI-mu.philowerCI)/2/1.96
  
        mu.phi = c(array(NA,numPops-1))
        phi = c(array(NA,numPops-1))
        for (chain in 1:nChain ) {
          # Allocate initial populations at random
          popID <-  sample(c(1:numPops),imig$numSample2[focalPop],replace=TRUE)
  
          # Pick extreme values for phi and mu.phi (more than 1sd from the mean)
          for (i in 1:(numPops-1)) {
            phi[i+1] <- rnorm(1,mean=mean.phi[i],sd=SD.phi[i])
            while ( abs(phi[i+1] - mean.phi[i]) < SD.phi[i]) {
              phi[i+1] <- rnorm(1,mean=mean.phi[i],sd=SD.phi[i])
            }
          }
          for (i in 1:(numPops-1)) {
            mu.phi[i] <- rnorm(1,mean=mean.mu[i],sd=SD.mu[i])
            while ( abs(mu.phi[i]-mean.mu[i])<SD.mu[i] ) {
                  mu.phi[i] <- rnorm(1,mean=mean.mu[i],sd=SD.mu[i])
            }
          }
  
          # We'll generate R using genInits to ensure it is overdispersed.
          modelInitVals <- list(z_tilde = popID, phi=phi, mu.phi=mu.phi)
  
          # Rewrite the Init data files
          initFile[[chain]] <- paste("imigModel_FocalPop",imig$demeNames[focalPop],"_Overdispersed_Inits",chain,".txt",sep="")
  
          bugsData(modelInitVals, initFile[[chain]],digits=5)
        }
  
        # Now reinitialise the Markov chain and do a new burn-in
        modelCheck(modelFile)      # Check model syntax
        dataFile <- paste("imigModel_FocalPop",imig$demeNames[focalPop],"_Data.txt",sep="")
        modelData(dataFile)        # Load in model data
        modelCompile(numChains=nChain) # Compile model
  
        for (chain in 1:nChain) {
          modelInits(initFile[[chain]], chainNum=chain)
        }
        # Generate the values for R node
        modelGenInits()
      }
      ##############################################################
  
      # Let markov chain go through adaptive phase
      # (by default for these distributions this is 4000 steps)
      modelUpdate(numUpdates=burninIters, thin=1, overRelax = FALSE)
  
      # Set variable to record output from MCMC
      samplesSet(paramSave)
  
      # Update model, now using thin parameter
      modelUpdate(numUpdates=datarecordIters, thin=thinParam, overRelax = FALSE)
  
      # Save results in CODA format
      for ( node in 1:length(paramSave) ) {
         # Remove . from the variable name
        paramName = paste(unlist(strsplit(paramSave[[node]], ".", fixed = TRUE)),sep='',collapse='')
        codaFile <- paste(outputPrefix,"_imigModel_FocalPop",imig$demeNames[focalPop],"_node",paramName,"_",sep="")
        samplesCoda(paramSave[[node]],codaFile)  # Generate data for CODA
      }
    }
    detach("package:BRugs")
}