  writeImigData <- function() {
  # A file to read in the genotype data (Fstat format) and reformat it
  # for use in the Bayesian pairwise migration rate model (Broquet, Yearsley et al 2009)
  #
  # Data for each population is saved in a separate file, imig_params.r
  # 
  # sample1Filename = The filename of the first sample (pre-dispersal)
  # sample2Filename = The filename of the second sample (post-dispersal)
  # demeGroups      = A vector specifying which group each population should be placed in. 
  #                   A group called 0 will be removed from the analysis
  #                   e.g. c(1,2,2,3,0) 5 populations with populations 2 and 3 grouped together
  #                   and opulation 5 not included in the analysis
  
  # Jon Yearsley  19th Jan 2009 jon.yearsley@ucd.ie
  #
  
  # Load some packages
  require(hierfstat, quietly=TRUE)
  require(BRugs, quietly=TRUE)
 
 # Read in the model parameters
 source('imig_params.r',local=TRUE)
 
 # Check the required variables have been defined in imig_params.r
 if (length(ls(pattern='demeGroups'))==0) {
  warning('demeGroups not defined in the file imig_params.r')
 }
 if (length(ls(pattern='sample1Filename'))==0) {
  warning('sample1Filename not defined in the file imig_params.r')
 }
 if (length(ls(pattern='sample2Filename'))==0) {
  warning('sample2Filename not defined in the file imig_params.r')
 }
 if (length(ls(pattern='mPrior_Mean'))==0) {
  warning('mPrior_Mean not defined in the file imig_params.r')
 }
 if (length(ls(pattern='mPrior_CV'))==0) {
  warning('mPrior_CV not defined in the file imig_params.r')
 }
 if (length(ls(pattern='nChain'))==0) {
  warning('nChain not defined in the file imig_params.r')
 }
 
  # Get initial info on number of populations and number of loci from sample 1 file
  line1 <- scan(sample1Filename,nmax=4, quiet=TRUE, what=integer(0))
  numPops_fstat1 <- line1[1] # Number of populations (first number of line 1 from fstat file)
  numLoci <- line1[2]    # Number of loci (second number in line 1 of fstat file)
  numDigit <- line1[4]  # Number of digits used to code an allele (forth number in line 1 of fstat file)

  # Check sample 2 file is consistent with sample 1 file
  line1 <- scan(sample2Filename,nmax=4, quiet=TRUE, what=integer(0))
  numPops_fstat2 <- line1[1] # Number of populations (first number of line 1 from fstat file)
  if (line1[2]!=numLoci) {
    warning('Differet number of loci in sample 1 and sample 2 fstat files') 
  }
  if (line1[4]!=numDigit) {
    warning('Differet number of digits used to code alleles in sample 1 and sample 2 fstat files') 
  }
  
  # If there are too few groups in demeGroups then remove all the groupings
  if ( length(demeGroups)<line1[1] )  {
    warning('Not the correct number of populations in demeGroups. Resetting the parameter demeGroups')
    demeGroups <- c(1:line1[1])
  }
  
  # Read in all the data for sample 1 and sample 2 individuals
  data1 <- read.fstat.data(sample1Filename,nloc=numLoci)
  data2 <- read.fstat.data(sample2Filename,nloc=numLoci)
  
  # Record which population each individual belongs to
  popId_raw1 <- data1$Pop
  popId_raw2 <- data2$Pop
  
  # Work out which demes have sample 1 and sample 2 data
  groupList1 <- demeGroups[popId_raw1]
  groupList2 <- demeGroups[popId_raw2]
    
  # Only use data where there is data in sample 1
  demeNames <- sort(unique(groupList1)) 

  # Collect together the popIDs for individuals who will be used in the analysis
  #
  # Transform these IDs into the groupings and remove individuals who are in demeGroup zero
  useSample1 <- is.element(groupList1,demeNames) & (groupList1 != 0)
  useSample2 <- is.element(groupList2,demeNames) & (groupList2 != 0)
  # Call these _tmp because we still have to rename the pop ID's so that they
  # run from 1 to numPops
  popId1_tmp <- groupList1[useSample1]
  popId2_tmp <- groupList2[useSample2]
  
  # Remove 0 groups from demeNames
  demeNames <- demeNames[demeNames != 0]

  # Set number of sample 1 and 2 individuals to be used in the analysis
  numSample1 <- length(popId1_tmp)
  numSample2 <- length(popId2_tmp)

  # Allocate arrays
  sample1Alleles <- array(0,dim=c(numSample1,numLoci,2))
  sample2Alleles <- array(0,dim=c(numSample2,numLoci,2))

  numAlleles <- c(array(0,c(numLoci)))
  uniqueAlleles <- c() # Define a NULL array
  
  # Record genotype... each allele is real or imaginary part
  # Homozygotes are real, heterozygotes have an imaginary part
  sample1Genotype <- array(0,dim=c(numSample1,numLoci))
  sample2Genotype <- array(0,dim=c(numSample2,numLoci))
  
  numGenotypes <- c(array(0,c(numLoci)))
  uniqueGenotypes <- c() # Define a NULL array

  # Select the indivduals which we are going to use and save the allele data  
  for ( locInd in 1:numLoci ) {    # Extract allele information
    tmp <- data1[useSample1, 1+locInd]
    sample1Alleles[, locInd, 1] <- array(tmp%/%10^numDigit, dim=numSample1)
    sample1Alleles[, locInd, 2] <- array(tmp%%10^numDigit, dim=numSample1)
    # Record sample 1 genotypes
    sample1Genotype[,locInd] =  sample1Alleles[ , locInd, 1];
    heteroInd = (sample1Alleles[, locInd, 1]<sample1Alleles[, locInd, 2])  # Index of heterozygotes
    sample1Genotype[heteroInd,locInd] =
       complex(real=sample1Alleles[heteroInd, locInd, 1],imag=sample1Alleles[heteroInd, locInd, 2]);
    heteroInd = (sample1Alleles[, locInd, 1]>sample1Alleles[, locInd, 2])  # Index of heterozygotes
    sample1Genotype[heteroInd,locInd] =
       complex(real=sample1Alleles[heteroInd, locInd, 2],imag=sample1Alleles[heteroInd, locInd, 1]);

    tmp <- data2[useSample2, 1+locInd]
    sample2Alleles[, locInd, 1] <- array(tmp%/%10^numDigit, dim=numSample2)
    sample2Alleles[, locInd, 2] <- array(tmp%%10^numDigit, dim=numSample2)
    # Record sample 2 genotypes
    sample2Genotype[,locInd] =  sample2Alleles[ , locInd, 1];
    heteroInd = (sample2Alleles[, locInd, 1]<sample2Alleles[, locInd, 2])  # Index of heterozygotes
    sample2Genotype[heteroInd,locInd] =
       complex(real=sample2Alleles[heteroInd, locInd, 1],imag=sample2Alleles[heteroInd, locInd, 2]);
    heteroInd = (sample2Alleles[, locInd, 1]>sample2Alleles[, locInd, 2])  # Index of heterozygotes
    sample2Genotype[heteroInd,locInd] =
       complex(real=sample2Alleles[heteroInd, locInd, 2],imag=sample2Alleles[heteroInd, locInd, 1]);

    # Calculate the unique alleles
    tmp <- sort( unique(array(c(sample2Alleles[,locInd,], sample1Alleles[,locInd,]), c(2*(numSample1+numSample2),1))))
    uniqueAlleles <- c(uniqueAlleles, list(tmp))

    # Record number of unique alleles
    numAlleles[locInd] <- length(tmp)

    # Calculate the unique genotypes at this locus
    tmp <- sort( unique(array(c(sample2Genotype[,locInd], sample1Genotype[,locInd]), c(numSample1+numSample2,1))))
    uniqueGenotypes <- c(uniqueGenotypes, list(tmp))
    # Number of unique genotypes
    numGenotypes[locInd] <- length(tmp) 
  }
  numAllelesMax <- max(numAlleles)
  numGenotypesMax <- max(numGenotypes)
  
  ##################################################################
  ##################################################################
  ##################################################################
  # Now put data in final format
  
  # Set number of populations to be number of populations with  data in sample 1
  # and take into account that these might be groupings
  numPops = length(demeNames)

  # Rename sample 1 and 2 pop ID's so that they run from 1 to numPops
  popId1 <- rep(NA,numSample1)
  popId2 <- rep(NA,numSample2)
  for ( p in 1:numPops) {
      popId1[popId1_tmp==demeNames[p]] <- demeNames[p]
      popId2[popId2_tmp==demeNames[p]] <- demeNames[p]
  }

  # First calculate genotype frequencies and genotype counts
  sample1GenotypeNum <- array(NA, c(numPops, numLoci, numGenotypesMax))
  sample2GenotypeNum <- array(NA, c(numPops, numLoci, numGenotypesMax))
  sample1GenotypeFreq <- array(0, c(numPops, numLoci, numGenotypesMax))
  swapInd <- array(NA, c(numPops, numPops))
 
  for ( popInd in 1:numPops ) {
    Ind1 <- (groupList1[useSample1] == demeNames[popInd])    # Sample 1 individuals who are in this group
    Ind2 <- (groupList2[useSample2] == demeNames[popInd])    # Sample 2 individuals who are in this group
    
    for ( locInd in 1:numLoci ) {    
      ########### Genotype counts
      for ( gtInd in 1:numGenotypes[locInd] ) {
        # Record number of each genotype at each locus in each population
        sample1GenotypeNum[popInd, locInd, gtInd] <- 
          sum(as.integer(Ind1 & sample1Genotype[,locInd]==uniqueGenotypes[[locInd]][gtInd]))
        sample2GenotypeNum[popInd, locInd, gtInd] <- 
          sum(as.integer(Ind2 & sample2Genotype[,locInd]==uniqueGenotypes[[locInd]][gtInd]))
     }

      #####################################
      # Calculate Genotype frequencies
      for ( gtInd in 1:numGenotypes[ locInd ] ) {
        genotypeSum <- sum(sample1GenotypeNum[ popInd, locInd, 1:numGenotypes[locInd] ] )
        sample1GenotypeFreq[popInd, locInd, gtInd] <- sample1GenotypeNum[ popInd, locInd, gtInd] / genotypeSum
      }
      # Add a small component to avoid zero frequencies
      sample1GenotypeFreq[popInd, locInd, ] <- 1.0e-12 + sample1GenotypeFreq[ popInd, locInd, ]
      # and renormalise again
      sample1GenotypeFreq[popInd, locInd,  ] <- sample1GenotypeFreq[ popInd, locInd, ]/ sum(sample1GenotypeFreq[ popInd, locInd, ])
    }
    
    # This is an array of indices which puts the phylopatric population in as
    # first element
    swapInd[popInd,1:numPops] <- c(1:numPops)
    swapInd[popInd,1] <- popInd
    swapInd[popInd,popInd] <- 1
  }

 
  # Now sample1 data... this is just a matter of removing unknown genotypes so that genotype
  # ID is 1, 2, 3 ...
  # sample1GenotypeData gives the genotypes present in each individual
  sample1GenotypeData <- array(NA,c(numSample1,numLoci))
  sample1GenotypeDataList <- vector(mode="list",c(numSample1,numLoci))
  for ( adInd in 1:numSample1 ) {
   for ( locInd in 1:numLoci ) {
    tmp <- c(1:numGenotypes[locInd])
    genotypeList <- uniqueGenotypes[[locInd]]  # List of all genotypes at this locus
    # Now rename the genotypes at this locus for this individual
    sample1GenotypeData[ adInd, locInd] <-  tmp[sample1Genotype[ adInd, locInd] == genotypeList]
   }
  }
                
  # Now sample2 data... this is just a matter of removing unknown genotypes so that genotype
  # ID is 1, 2, 3 ...
  # sample2GenotypeData gives the genotype present in each sample 2 individual
  sample2GenotypeData <- array(NA,c(numSample2,numLoci))
  sample2GenotypeDataList <- vector(mode="list",c(numSample2*numLoci))
  for ( adInd in 1:numSample2 ) {
   for ( locInd in 1:numLoci ) {
    tmp <- c(1:numGenotypes[locInd])
    genotypeList <- uniqueGenotypes[[locInd]]  # List of all genotypes at this locus
    # Now rename the genotypes at this locus for this individual
    sample2GenotypeData[ adInd, locInd] <-  tmp[sample2Genotype[ adInd, locInd] == genotypeList]
   }
  }

  # Finally calculate sample2 genotype frequencies for use in the linear model
  sample2GenotypeFreq <- array(NA,c(numPops,numLoci,numGenotypesMax))
  for ( popInd in 1:numPops ) {
      # Only take individuals who are in demes to be used in analysis
      Ind2 <- (groupList2[useSample2] == demeNames[popInd])
      for (locInd in 1:numLoci ) {
           for ( gtInd in 1:numGenotypes[locInd] ) {
             sample2GenotypeFreq[ popInd, locInd, gtInd] <-  sum(as.integer(sample2Genotype[Ind2,locInd]==uniqueGenotypes[[locInd]][gtInd]))
             }
             sample2GenotypeFreq[ popInd, locInd, ] <- sample2GenotypeFreq[ popInd, locInd, ] / sum(sample2GenotypeFreq[ popInd, locInd, 1:numGenotypes[locInd]])
      }
  }
 
  ####################################################
  # Now set up data for the openBugs model
  # Write a data file and initial value files for each focal population
  
  # Record important info in dummy variables
  popId2_tmp <- popId2
  sample2GenotypeData_tmp <- sample2GenotypeData
  sample2GenotypeNum_tmp <- sample2GenotypeNum
  sample2GenotypeFreq_tmp <- sample2GenotypeFreq
  swapInd_tmp <- swapInd
  numSample2 = array(0,dim=numPops)

  # Define number of populations minus 1!
  nPm <- numPops - 1
 
  for ( focalPop in 1:numPops ) {
    # Write data for each focal populations
  
    # Find relevant sample 2 individuals for this focal population
    adInd <- (popId2_tmp == demeNames[focalPop])
    numSample2[focalPop] <- sum(adInd)
    popId2 <- popId2_tmp[adInd]
    sample2GenotypeData <- sample2GenotypeData_tmp[adInd,]
    sample2GenotypeNum <- sample2GenotypeNum_tmp[focalPop, , ]
    sample2GenotypeFreq <- sample2GenotypeFreq_tmp[focalPop, , ]
    swapInd <- swapInd_tmp[focalPop,]

    # Prior mean and precision for phi
    # Calculate in terms of m (prob of migrating) so that 1-m is prob of philopatry
    # Philopatry is by default given a phi = 5 in the model
    mu.phi_mean <- 5 + log(mPrior_Mean / (1-mPrior_Mean)) - log(numPops-1)
    mu.phi_var <- (numPops-1)^2 * exp(2*mu.phi_mean) * mPrior_CV^2
    mean.mu <- c(array(mu.phi_mean,dim=(numPops - 1)))
    prec.mu <- diag(1/mu.phi_var,numPops-1)

    # Data for Wishart distribution.  (see Billheimer 2001 for Nprime)
    Nprime <- matrix(1, numPops-1,numPops-1) + diag(1,numPops-1)
    a <- 0.1
    Omega <- a*Nprime
   
    # Write data to files
    dataFile <- paste("imigModel_FocalPop",demeNames[focalPop],"_Data.txt",sep="")
   
    # Data for estimating pairwise migration rates
    modelDataNames <- list(numPops=numPops,nPm=nPm,numLoci=numLoci,numSample2=numSample2[focalPop],
       sample1GenotypeFreq=sample1GenotypeFreq,sample2GenotypeData=sample2GenotypeData,
       swapInd=swapInd,mean.mu=mean.mu,prec.mu=prec.mu,Omega=Omega)
    bugsData(modelDataNames,dataFile,digits=5)

    # List of initialisation variables for the model
    initFile <- vector("list",nChain)
  
    # Write initial data for each chain
    for ( chain in 1:nChain ) {
        # Allocate initial populations at random
        popID <-  sample(c(1:numPops),numSample2[focalPop],replace=TRUE)
  
        # Set initial values of phi at random
        phi <- rnorm(numPops,mean=0,sd=0.0001)
        phi[1] <- NA  # This phi[1] is constant at 5 (philopatry)

        mu.phi <- c(array(0,numPops - 1))
        Sigma.phi <- diag(0.1, numPops - 1)

        modelInitVals <- list(z_tilde=popID, phi=phi, Sigma.phi=Sigma.phi, mu.phi=mu.phi)        
        initFile[chain] <- paste("imigModel_FocalPop",demeNames[focalPop],"_Inits",chain,".txt",sep="")
        bugsData(modelInitVals, initFile[[chain]],digits=5)
    }
  }
  detach("package:BRugs")
  detach("package:hierfstat")

  return(list(demeNames=demeNames,numSample1=numSample1,numSample2=numSample2))
}

